package dev.adriankuta.pixabay.data.room.entity

import androidx.room.ColumnInfo
import androidx.room.Entity
import androidx.room.PrimaryKey
import dev.adriankuta.pixabay.data.dto.response.PixabayImageResponse
import dev.adriankuta.pixabay.data.room.entity.PixabayImageEntity.Companion.TABLE_NAME

@Entity(tableName = TABLE_NAME)
internal data class PixabayImageEntity(
    @PrimaryKey(autoGenerate = true)
    @ColumnInfo("entry_id") val entryId: Int = 0,
    @ColumnInfo(COLUMN_ID) val id: Int,
    @ColumnInfo(COLUMN_QUERY_USED) val queryUsed: String,
    @ColumnInfo(COLUMN_PAGE_URL) val pageURL: String,
    @ColumnInfo(COLUMN_TYPE) val type: String,
    @ColumnInfo(COLUMN_TAGS) val tags: String,
    @ColumnInfo(COLUMN_PREVIEW_URL) val previewURL: String,
    @ColumnInfo(COLUMN_PREVIEW_WIDTH) val previewWidth: Int,
    @ColumnInfo(COLUMN_PREVIEW_HEIGHT) val previewHeight: Int,
    @ColumnInfo(COLUMN_WEB_FORMAT_URL) val webformatURL: String,
    @ColumnInfo(COLUMN_WEB_FORMAT_WIDTH) val webformatWidth: Int,
    @ColumnInfo(COLUMN_WEB_FORMAT_HEIGHT) val webformatHeight: Int,
    @ColumnInfo(COLUMN_LARGE_IMAGE_URL) val largeImageURL: String,
    @ColumnInfo(COLUMN_IMAGE_WIDTH) val imageWidth: Int,
    @ColumnInfo(COLUMN_IMAGE_HEIGHT) val imageHeight: Int,
    @ColumnInfo(COLUMN_IMAGE_SIZE) val imageSize: Int,
    @ColumnInfo(COLUMN_VIEWS) val views: Int,
    @ColumnInfo(COLUMN_DOWNLOADS) val downloads: Int,
    @ColumnInfo(COLUMN_LIKES) val likes: Int,
    @ColumnInfo(COLUMN_COMMENTS) val comments: Int,
    @ColumnInfo(COLUMN_USER_ID) val userId: Int,
    @ColumnInfo(COLUMN_USER) val user: String,
    @ColumnInfo(COLUMN_USER_IMAGE_URL) val userImageURL: String,

) {
    constructor(pixabayImageResponse: PixabayImageResponse, queryUsed: String) :
            this(
                id = pixabayImageResponse.id,
                queryUsed = queryUsed,
                pageURL = pixabayImageResponse.pageURL,
                type = pixabayImageResponse.type,
                tags = pixabayImageResponse.tags,
                previewURL = pixabayImageResponse.previewURL,
                previewWidth = pixabayImageResponse.previewWidth,
                previewHeight = pixabayImageResponse.previewHeight,
                webformatURL = pixabayImageResponse.webformatURL,
                webformatWidth = pixabayImageResponse.webformatWidth,
                webformatHeight = pixabayImageResponse.webformatHeight,
                largeImageURL = pixabayImageResponse.largeImageURL,
                imageWidth = pixabayImageResponse.imageWidth,
                imageHeight = pixabayImageResponse.imageHeight,
                imageSize = pixabayImageResponse.imageSize,
                views = pixabayImageResponse.views,
                downloads = pixabayImageResponse.downloads,
                likes = pixabayImageResponse.likes,
                comments = pixabayImageResponse.comments,
                userId = pixabayImageResponse.user_id,
                user = pixabayImageResponse.user,
                userImageURL = pixabayImageResponse.userImageURL
            )


    companion object {
        const val TABLE_NAME = "pixabay_images"
        const val COLUMN_ID = "id"
        const val COLUMN_QUERY_USED = "query_used"
        const val COLUMN_PAGE_URL = "page_url"
        const val COLUMN_TYPE = "type"
        const val COLUMN_TAGS = "tags"
        const val COLUMN_PREVIEW_URL = "preview_url"
        const val COLUMN_PREVIEW_WIDTH = "preview_width"
        const val COLUMN_PREVIEW_HEIGHT = "preview_height"
        const val COLUMN_WEB_FORMAT_URL = "web_format_url"
        const val COLUMN_WEB_FORMAT_WIDTH = "web_format_width"
        const val COLUMN_WEB_FORMAT_HEIGHT = "web_format_height"
        const val COLUMN_LARGE_IMAGE_URL = "large_image_url"
        const val COLUMN_IMAGE_WIDTH = "image_width"
        const val COLUMN_IMAGE_HEIGHT = "image_height"
        const val COLUMN_IMAGE_SIZE = "image_size"
        const val COLUMN_VIEWS = "views"
        const val COLUMN_DOWNLOADS = "downloads"
        const val COLUMN_LIKES = "likes"
        const val COLUMN_COMMENTS = "comments"
        const val COLUMN_USER_ID = "user_id"
        const val COLUMN_USER = "user"
        const val COLUMN_USER_IMAGE_URL = "user_image_url"
    }
}