@file:OptIn(ExperimentalCoroutinesApi::class, FlowPreview::class)

package dev.adriankuta.pixabay.feature.search

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import androidx.paging.PagingData
import androidx.paging.cachedIn
import dagger.hilt.android.lifecycle.HiltViewModel
import dev.adriankuta.pixabay.data.model.PixabayImage
import dev.adriankuta.pixabay.data.repository.ImageRepository
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.FlowPreview
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.debounce
import kotlinx.coroutines.flow.flatMapLatest
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.stateIn
import javax.inject.Inject

@HiltViewModel
class SearchScreenViewModel @Inject constructor(
    private val imageRepository: ImageRepository,
) : ViewModel() {

    private val _query = MutableStateFlow(INITIAL_QUERY)
    val state: StateFlow<SearchUiState>

    val pagingDataFlow: Flow<PagingData<PixabayImage>>

    init {
        pagingDataFlow = _query
            .debounce(300)
            .flatMapLatest { searchImages(query = it) }
            .cachedIn(viewModelScope)

        state = _query
            .map { search ->
                SearchUiState(query = search)
            }
            .stateIn(
                scope = viewModelScope,
                started = SharingStarted.WhileSubscribed(stopTimeoutMillis = 5000),
                initialValue = SearchUiState()
            )
    }

    fun onQueryChange(query: String) {
        _query.value = query
    }

    private fun searchImages(query: String): Flow<PagingData<PixabayImage>> =
        imageRepository.getSearchResultStream(query)

    companion object {
        const val INITIAL_QUERY = "fruits"
    }
}